<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Providers\RouteServiceProvider;
use App\User;
use Illuminate\Foundation\Auth\RegistersUsers;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Hash;
use App\Mail\SignUp;
use Illuminate\Support\Facades\Validator;
use Auth;

class RegisterController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Register Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles the registration of new users as well as their
    | validation and creation. By default this controller uses a trait to
    | provide this functionality without requiring any additional code.
    |
    */

     use RegistersUsers;

    /**
     * Where to redirect users after registration.
     *
     * @var string
     */
      protected $redirectTo = RouteServiceProvider::HOME;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('guest');
    }

    /**
     * Get a validator for an incoming registration request.
     *
     * @param  array  $data
     * @return \Illuminate\Contracts\Validation\Validator
     */
    protected function validator(array $data)
    {
        if($data['user_type']==1){
            return Validator::make($data, [
                'givenname' => ['required', 'string', 'max:255'],
                'familyname' => ['required', 'string', 'max:255'],
                'email' => ['required', 'string', 'email', 'max:255', 'unique:users'],
                'password' => ['required', 'string', 'min:8', 'confirmed'],
                'nationality' => ['required', 'string','max:255'],
                'dob' => ['required', 'string','date'],
                'optradio' => ['required', 'string','max:255'],
            ]);
        } 
        
    }

    /**
     * Create a new user instance after a valid registration.
     *
     * @param  array  $data
     * @return \App\User
     */
    protected function create(array $data)
    { 
        if($data['user_type']==1){
            $data = User::create([
                'first_name' => $data['givenname'],
                'last_name' => $data['familyname'],
                'email' => $data['email'],
                'nationality' => $data['nationality'],
                'dob' => \Carbon\Carbon::parse($data['dob'])->format('Y-m-d'),
                'gender' => $data['optradio'],
                'user_type' => $data['user_type'],
                'password' => Hash::make($data['password'])
            ]);
            Mail::to('test@mail.com')->send(new SignUp($data->id));

        }  
        
        if($data['user_type']==1){
           $this->redirectTo="/home";
        } 

        return $data;

    }

    public function email_verification($id)
     {
        $data = User::find($id);
        $data->email_verification=1;
        $data->update();
        return Redirect('/login')->with('success','Your mail is activated successfully. Login here ');
     }
}
